import { Component, HostListener, OnInit } from '@angular/core';
import { Structure } from '../../models/structure.model';
import { User } from '../../models/user.model';
import { OrientationService } from '../../services/orientation.service';
import { StructureService } from '../../services/structure.service';
import { ProfileService } from '../services/profile.service';

enum tabsEnum {
  myOrientations,
  todoOrientations,
}

@Component({
  selector: 'app-dashboard',
  templateUrl: './dashboard.component.html',
  styleUrls: ['./dashboard.component.scss'],
})
export class DashboardComponent implements OnInit {
  constructor(
    private orientationService: OrientationService,
    private structureService: StructureService,
    private profileService: ProfileService,
  ) {}
  public orientationsDashboard;
  public userProfile: User;
  public structures = [];
  public tabsEnum = tabsEnum;
  public currentTab = tabsEnum.todoOrientations;
  public options = [];
  public isOpen = false;
  public selectedOption;

  ngOnInit(): void {
    this.getOrientationsDashboard();
    this.profileService.getProfile().then((profile) => {
      this.userProfile = new User(profile);
      this.getStructuresFromProfile();
    });
  }

  // TABS
  public getTabsNames(): string[] {
    return ["Suivi d'orientation", 'Accompagnements à faire'];
  }
  public getMobileTabsNames(): string[] {
    return ['Suivi', 'A faire'];
  }

  public getTabsIds(): string[] {
    return ['myOrientations', 'todoOrientations'];
  }

  public changeTab(tab: number): void {
    this.currentTab = tab;
  }

  // CUSTOM SELECT
  @HostListener('document:click', ['$event'])
  public onClick(event: MouseEvent): void {
    if (!(event.target as HTMLElement).closest('.custom-select')) {
      this.isOpen = false;
    }
  }

  private getOrientationsDashboard(structureId = ''): void {
    this.orientationService.getOrientationsDashboard(structureId).subscribe((orientationsDashboard) => {
      this.orientationsDashboard = orientationsDashboard;

      // Stay on the current tab unless it has no inProgress orientations and the other tab has ones (with todoOrientations as the default tab)
      const { todoOrientations, myOrientations } = this.orientationsDashboard;
      if (todoOrientations.inProgress.length && !myOrientations.inProgress.length) {
        this.changeTab(tabsEnum.todoOrientations);
      } else if (!todoOrientations.inProgress.length && myOrientations.inProgress.length) {
        this.changeTab(tabsEnum.myOrientations);
      }
    });
  }

  private getStructuresFromProfile(): void {
    const fetchStructures = this.userProfile.structuresLink.map((structureId) =>
      this.structureService.getStructureWithOwners(structureId).toPromise(),
    );
    Promise.all(fetchStructures).then((structures) => {
      this.structures = structures
        .map((s) => new Structure(s.structure))
        .sort((a, b) => a.structureName.localeCompare(b.structureName));
      this.updateOptions();
    });
  }

  private updateOptions(): void {
    this.options = [
      { id: '', name: 'Toutes mes structures', icon: 'allStructures' },
      ...this.structures.map((structure) => ({
        id: structure._id,
        name: structure.structureName,
        icon: structure.getTypeStructureIcon(),
      })),
    ];
    // default option
    if (!this.selectedOption || this.selectedOption.id === 'all') {
      this.selectedOption = this.options[0];
    }
  }

  public toggleDropdown(): void {
    this.isOpen = !this.isOpen;
  }

  public onKeyDown(event: KeyboardEvent): void {
    if (event.key === ' ' || event.key === 'Enter') {
      event.preventDefault();
      this.toggleDropdown();
    }
  }

  public selectOption(option: { id: string; name: string; icon: string }): void {
    this.getOrientationsDashboard(option.id);
    this.selectedOption = option;
    this.isOpen = false;
  }

  public onSortChange(orientations: any[]): void {
    orientations.reverse();
  }
}
