import { Component, Input, OnInit } from '@angular/core';
import { Structure } from '../../../models/structure.model';
import { FreeWorkShopLabels } from '../../../structure/enums/freeWorkshop.enum';

@Component({
  selector: 'app-print-structures-grid',
  templateUrl: './print-structures-grid.component.html',
  styleUrls: ['./print-structures-grid.component.scss'],
})
export class PrintStructuresGridComponent implements OnInit {
  @Input({ required: true }) structures: Structure[];
  public structuresByCommunePairs: Record<string, Structure[][]>;

  public FreeWorkShopLabels = FreeWorkShopLabels;

  ngOnInit(): void {
    this.structuresByCommunePairs = this.structures.reduce((communes, structure) => {
      const communePairs: Structure[][] = communes[structure.address.commune] || [];

      if (communePairs.length === 0 || communePairs[communePairs.length - 1].length === 2) {
        communePairs.push([structure]);
      } else {
        communePairs[communePairs.length - 1].push(structure);
      }

      communes[structure.address.commune] = communePairs;
      return communes;
    }, {});
  }

  compareKeys(a: { key: string }, b: { key: string }): number {
    // Use localeCompare to sort the keys to avoid Écully with accent to be after all the other cities
    return a.key.localeCompare(b.key);
  }

  getServices(structure: Structure): string[] {
    const services: string[] = [];
    if (structure.hasFreeOnlineHelp()) {
      services.push(`Aide aux démarches en ligne : ${FreeWorkShopLabels.free.toLowerCase()}`);
    }
    if (structure.hasWorkShopHelp()) {
      services.push(`Accompagnement aux usages numériques : ${structure.getFreeWorkshopLabels().toLowerCase()}`);
    }
    if (structure.hasSolidarityMaterial()) {
      services.push('Matériel à tarif solidaire');
    }
    if (structure.hasEquipments()) {
      services.push('Matériel en accès libre');
    }
    return services;
  }
}
